/* SPDX-License-Identifier: LGPL-2.1-or-later */
/*
 * Copyright (C) 2020, Google Inc.
 *
 * vimc_ipa_proxy.cpp - Image Processing Algorithm proxy for vimc
 *
 * This file is auto-generated. Do not edit.
 */

#include <libcamera/ipa/vimc_ipa_proxy.h>

#include <memory>
#include <string>
#include <vector>

#include <libcamera/ipa/ipa_module_info.h>
#include <libcamera/ipa/vimc_ipa_interface.h>
#include <libcamera/ipa/vimc_ipa_serializer.h>

#include <libcamera/base/log.h>
#include <libcamera/base/thread.h>

#include "libcamera/internal/control_serializer.h"
#include "libcamera/internal/ipa_data_serializer.h"
#include "libcamera/internal/ipa_module.h"
#include "libcamera/internal/ipa_proxy.h"
#include "libcamera/internal/ipc_pipe.h"
#include "libcamera/internal/ipc_pipe_unixsocket.h"
#include "libcamera/internal/ipc_unixsocket.h"
#include "libcamera/internal/process.h"

namespace libcamera {

LOG_DECLARE_CATEGORY(IPAProxy)

namespace ipa {

namespace vimc {


IPAProxyVimc::IPAProxyVimc(IPAModule *ipam, bool isolate)
	: IPAProxy(ipam), isolate_(isolate), seq_(0)
{
	LOG(IPAProxy, Debug)
		<< "initializing vimc proxy: loading IPA from "
		<< ipam->path();

	if (isolate_) {
		const std::string proxyWorkerPath = resolvePath("vimc_ipa_proxy");
		if (proxyWorkerPath.empty()) {
			LOG(IPAProxy, Error)
				<< "Failed to get proxy worker path";
			return;
		}

		ipc_ = std::make_unique<IPCPipeUnixSocket>(ipam->path().c_str(),
							   proxyWorkerPath.c_str());
		if (!ipc_->isConnected()) {
			LOG(IPAProxy, Error) << "Failed to create IPCPipe";
			return;
		}

		ipc_->recv.connect(this, &IPAProxyVimc::recvMessage);

		valid_ = true;
		return;
	}

	if (!ipam->load())
		return;

	IPAInterface *ipai = ipam->createInterface();
	if (!ipai) {
		LOG(IPAProxy, Error)
			<< "Failed to create IPA context for " << ipam->path();
		return;
	}

	ipa_ = std::unique_ptr<IPAVimcInterface>(static_cast<IPAVimcInterface *>(ipai));
	proxy_.setIPA(ipa_.get());


	ipa_->dummyEvent.connect(this, &IPAProxyVimc::dummyEventThread);

	valid_ = true;
}

IPAProxyVimc::~IPAProxyVimc()
{
	if (isolate_) {
		IPCMessage::Header header =
			{ static_cast<uint32_t>(_VimcCmd::Exit), seq_++ };
		IPCMessage msg(header);
		ipc_->sendAsync(msg);
	}
}


void IPAProxyVimc::recvMessage(const IPCMessage &data)
{
	size_t dataSize = data.data().size();
	_VimcEventCmd _cmd = static_cast<_VimcEventCmd>(data.header().cmd);

	switch (_cmd) {
	case _VimcEventCmd::DummyEvent: {
		dummyEventIPC(data.data().cbegin(), dataSize, data.fds());
		break;
	}
	default:
		LOG(IPAProxy, Error) << "Unknown command " << static_cast<uint32_t>(_cmd);
	}
}


int32_t IPAProxyVimc::init(
	const IPASettings &settings)
{
	if (isolate_)
		return initIPC(settings);
	else
		return initThread(settings);
}

int32_t IPAProxyVimc::initThread(
	const IPASettings &settings)
{
	int32_t _ret = ipa_->init(settings);

	proxy_.moveToThread(&thread_);

	return _ret;
}

int32_t IPAProxyVimc::initIPC(
	const IPASettings &settings)
{
	IPCMessage::Header _header = { static_cast<uint32_t>(_VimcCmd::Init), seq_++ };
	IPCMessage _ipcInputBuf(_header);
	IPCMessage _ipcOutputBuf;


	std::vector<uint8_t> settingsBuf;
	std::tie(settingsBuf, std::ignore) =
		IPADataSerializer<IPASettings>::serialize(settings);
	_ipcInputBuf.data().insert(_ipcInputBuf.data().end(), settingsBuf.begin(), settingsBuf.end());


	int _ret = ipc_->sendSync(_ipcInputBuf, &_ipcOutputBuf);
	if (_ret < 0) {
		LOG(IPAProxy, Error) << "Failed to call init";
		return static_cast<int32_t>(_ret);
	}

	int32_t _retValue = IPADataSerializer<int32_t>::deserialize(_ipcOutputBuf.data(), 0);






	return _retValue;

}


int32_t IPAProxyVimc::start()
{
	if (isolate_)
		return startIPC();
	else
		return startThread();
}

int32_t IPAProxyVimc::startThread()
{
	state_ = ProxyRunning;
	thread_.start();

	return proxy_.invokeMethod(&ThreadProxy::start, ConnectionTypeBlocking);
}

int32_t IPAProxyVimc::startIPC()
{
	IPCMessage::Header _header = { static_cast<uint32_t>(_VimcCmd::Start), seq_++ };
	IPCMessage _ipcInputBuf(_header);
	IPCMessage _ipcOutputBuf;




	int _ret = ipc_->sendSync(_ipcInputBuf, &_ipcOutputBuf);
	if (_ret < 0) {
		LOG(IPAProxy, Error) << "Failed to call start";
		return static_cast<int32_t>(_ret);
	}

	int32_t _retValue = IPADataSerializer<int32_t>::deserialize(_ipcOutputBuf.data(), 0);






	return _retValue;

}


void IPAProxyVimc::stop()
{
	if (isolate_)
		stopIPC();
	else
		stopThread();
}

void IPAProxyVimc::stopThread()
{
	ASSERT(state_ != ProxyStopping);
	if (state_ != ProxyRunning)
		return;

	state_ = ProxyStopping;

	proxy_.invokeMethod(&ThreadProxy::stop, ConnectionTypeBlocking);

	thread_.exit();
	thread_.wait();

	Thread::current()->dispatchMessages(Message::Type::InvokeMessage);

	state_ = ProxyStopped;
}

void IPAProxyVimc::stopIPC()
{
	IPCMessage::Header _header = { static_cast<uint32_t>(_VimcCmd::Stop), seq_++ };
	IPCMessage _ipcInputBuf(_header);




	int _ret = ipc_->sendSync(_ipcInputBuf);
	if (_ret < 0) {
		LOG(IPAProxy, Error) << "Failed to call stop";
		return;
	}
}




void IPAProxyVimc::dummyEventThread(
	const uint32_t val)
{
	ASSERT(state_ != ProxyStopped);
	dummyEvent.emit(val);
}

void IPAProxyVimc::dummyEventIPC(
	std::vector<uint8_t>::const_iterator data,
	size_t dataSize,
	[[maybe_unused]] const std::vector<int32_t> &fds)
{
	uint32_t val;


	const size_t valStart = 0;


	val = IPADataSerializer<uint32_t>::deserialize(
        	data + valStart,
        	data + dataSize);

	dummyEvent.emit(val);
}


} /* namespace vimc */

} /* namespace ipa */

} /* namespace libcamera */